// ------------------------------------------------------------------------------
// TUniCheckListBox Component
//
// Developed By:  Andrea Arilotta
//
// Email: arilotta@medarchiver.com
//
// Last Update: 2017-08
// Unigui Version 1.0.0.1401. Delpi 10 Seattle
// ------------------------------------------------------------------------------
//
// Free to use
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//
// ------------------------------------------------------------------------------
//
unit Unichecklst;

interface

uses
  Classes, SysUtils, uniListBox, uniGUITypes
  ;

type
  TUniCheckListBox = class(TUniCustomListBox)
  private
    procedure SynchronizeJSCheck(Index: integer=-1);
  protected
    procedure JSEventHandler(AEventName:String;AParams:TUniStrings); override;
    procedure WebCreate; override;
    procedure SetSelected(Index: Integer; Value: Boolean);
    procedure DoListChanged; override;
  public
    constructor Create(AOwner:TComponent); override;
    procedure SelectAll;
    procedure ClearSelection;

    property SelCount;
    property Selected[Index: Integer]: Boolean read GetSelected write SetSelected;
  published
    property Items;
    property ItemIndex;
    property Align;
    property Anchors;
    property TabOrder;
    property TabStop;
    property ParentFont;
    property Font;
    property Sorted;
    property BorderStyle default ubsDefault;
    property ClientEvents;
    property ScreenMask;
    property LayoutConfig;
    property Draggable;

    property OnEndDrag;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnExit;
    property OnEnter;
    property OnKeyDown;
    property OnKeyUp;
    property OnKeyPress;
    property OnAjaxEvent;
    property OnMouseUp;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
  end;

procedure register;

implementation

uses System.Math;

procedure register;
begin
  RegisterComponents('uniGUI Custom', [TUniCheckListBox]);
end;

{ TUniCheckListBox }

procedure TUniCheckListBox.ClearSelection;
begin
  inherited;
  JSInterface.JSCall('boundList.refresh', []);
end;

constructor TUniCheckListBox.Create(AOwner: TComponent);
begin
  inherited;
  MultiSelect:=true;
end;

procedure TUniCheckListBox.DoListChanged;
begin
  inherited;
  ClearSelection;
end;

procedure TUniCheckListBox.JSEventHandler(AEventName: String;
  AParams: TUniStrings);
var idx: integer;
begin
  if SameText(AEventName, 'checkSelect') then
    begin
      idx:=StrToInt(AParams.Values['indx']);
      Selected[idx]:=not Selected[idx];
    end;
  inherited;
end;

procedure TUniCheckListBox.SelectAll;
var i: integer;
begin
  inherited;
  SynchronizeJSCheck;
end;

procedure TUniCheckListBox.SetSelected(Index: Integer; Value: Boolean);
var lApplyJS: boolean;
begin
  lApplyJS:=(Selected[Index] and (not Value)) or (not Selected[Index] and (Value));
  inherited;
  if lApplyJS then
    SynchronizeJSCheck(Index);
end;

procedure TUniCheckListBox.SynchronizeJSCheck(Index: integer);
var i: integer;
begin
  for i:=IfThen(Index=-1, 0, Index) to IfThen(Index=-1, Count-1, Index) do // nth(n) (1 based)
    JSInterface.JSCode(#1'.el.select("input[type=checkbox]:nth('+(i+1).ToString+')").elements[0].checked='+Selected[i].ToString.ToLower+';');
end;

procedure TUniCheckListBox.WebCreate;
var JSFunctionBody: string;
begin
  inherited;
  JSFunctionBody :=
  '  var me = sender,'+
  '      list = sender.boundList;'+
  ''+
  '  document._chSelect = function(cmp, indx) {'+
  '      ajaxRequest(cmp, "checkSelect", ["indx=" + (indx - 1)]);'+
  '  };'+
  ''+
  '  list.tpl.html = ''<ul class="x-list-plain"><tpl for="."><li role="option" unselectable="on" class="x-boundlist-item" style="cursor: default;">'+
                      '<input id={[Ext.id()]} type="checkbox" style="vertical-align:middle; cursor: pointer; cursor: hand;" onclick="_chSelect('+self.JSName+',{#})" />{val}</li></tpl></ul>'';'+
  ''+
  '  list.addListener(''beforeitemclick'', function() {'+
  '      return false'+
  '  });'+
  ''+
  '  list.addListener(''beforecontainerclick'', function() {'+
  '      return false'+
  '  })';
  JSAddListener('beforerender',
      JSFunction('sender, eOpts',  JSFunctionBody)
  );
end;

end.
